@@ -10,7 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import testtools
+from openstack.tests.unit import base
 
 from openstack.network import version
 
@@ -22,7 +22,7 @@ EXAMPLE = {
 }
 
 
-class TestVersion(testtools.TestCase):
+class TestVersion(base.TestCase):
 
     def test_basic(self):
         sot = version.Version()
