@@ -11,7 +11,7 @@
 # under the License.
 
 import mock
-import testtools
+from openstack.tests.unit import base
 
 from openstack import exceptions
 from openstack import proxy
@@ -42,10 +42,10 @@ class HeadableResource(resource.Resource):
     allow_head = True
 
 
-class Test_check_resource(testtools.TestCase):
+class TestProxyPrivate(base.TestCase):
 
     def setUp(self):
-        super(Test_check_resource, self).setUp()
+        super(TestProxyPrivate, self).setUp()
 
         def method(self, expected_type, value):
             return value
@@ -53,26 +53,28 @@ class Test_check_resource(testtools.TestCase):
         self.sot = mock.Mock()
         self.sot.method = method
 
+        self.fake_proxy = proxy.Proxy("session")
+
     def _test_correct(self, value):
         decorated = proxy._check_resource(strict=False)(self.sot.method)
         rv = decorated(self.sot, resource.Resource, value)
 
         self.assertEqual(value, rv)
 
-    def test_correct_resource(self):
+    def test__check_resource_correct_resource(self):
         res = resource.Resource()
         self._test_correct(res)
 
-    def test_notstrict_id(self):
+    def test__check_resource_notstrict_id(self):
         self._test_correct("abc123-id")
 
-    def test_strict_id(self):
+    def test__check_resource_strict_id(self):
         decorated = proxy._check_resource(strict=True)(self.sot.method)
         self.assertRaisesRegex(ValueError, "A Resource must be passed",
                                decorated, self.sot, resource.Resource,
                                "this-is-not-a-resource")
 
-    def test_incorrect_resource(self):
+    def test__check_resource_incorrect_resource(self):
         class OneType(resource.Resource):
             pass
 
@@ -85,8 +87,78 @@ class Test_check_resource(testtools.TestCase):
                                "Expected OneType but received AnotherType",
                                decorated, self.sot, OneType, value)
 
+    def test__get_uri_attribute_no_parent(self):
+        class Child(resource.Resource):
+            something = resource.Body("something")
 
-class TestProxyDelete(testtools.TestCase):
+        attr = "something"
+        value = "nothing"
+        child = Child(something=value)
+
+        result = self.fake_proxy._get_uri_attribute(child, None, attr)
+
+        self.assertEqual(value, result)
+
+    def test__get_uri_attribute_with_parent(self):
+        class Parent(resource.Resource):
+            pass
+
+        value = "nothing"
+        parent = Parent(id=value)
+
+        result = self.fake_proxy._get_uri_attribute("child", parent, "attr")
+
+        self.assertEqual(value, result)
+
+    def test__get_resource_new(self):
+        value = "hello"
+        fake_type = mock.Mock(spec=resource.Resource)
+        fake_type.new = mock.Mock(return_value=value)
+        attrs = {"first": "Brian", "last": "Curtin"}
+
+        result = self.fake_proxy._get_resource(fake_type, None, **attrs)
+
+        fake_type.new.assert_called_with(**attrs)
+        self.assertEqual(value, result)
+
+    def test__get_resource_from_id(self):
+        id = "eye dee"
+        value = "hello"
+        attrs = {"first": "Brian", "last": "Curtin"}
+
+        # The isinstance check needs to take a type, not an instance,
+        # so the mock.assert_called_with method isn't helpful here since
+        # we can't pass in a mocked object. This class is a crude version
+        # of that same behavior to let us check that `new` gets
+        # called with the expected arguments.
+
+        class Fake(object):
+            call = {}
+
+            @classmethod
+            def new(cls, **kwargs):
+                cls.call = kwargs
+                return value
+
+        result = self.fake_proxy._get_resource(Fake, id, **attrs)
+
+        self.assertDictEqual(dict(id=id, **attrs), Fake.call)
+        self.assertEqual(value, result)
+
+    def test__get_resource_from_resource(self):
+        res = mock.Mock(spec=resource.Resource)
+        res._update = mock.Mock()
+
+        attrs = {"first": "Brian", "last": "Curtin"}
+
+        result = self.fake_proxy._get_resource(resource.Resource,
+                                               res, **attrs)
+
+        res._update.assert_called_once_with(**attrs)
+        self.assertEqual(result, res)
+
+
+class TestProxyDelete(base.TestCase):
 
     def setUp(self):
         super(TestProxyDelete, self).setUp()
@@ -98,16 +170,16 @@ class TestProxyDelete(testtools.TestCase):
         self.res.id = self.fake_id
         self.res.delete = mock.Mock()
 
-        self.sot = proxy.BaseProxy(self.session)
-        DeleteableResource.existing = mock.Mock(return_value=self.res)
+        self.sot = proxy.Proxy(self.session)
+        DeleteableResource.new = mock.Mock(return_value=self.res)
 
     def test_delete(self):
         self.sot._delete(DeleteableResource, self.res)
-        self.res.delete.assert_called_with(self.session)
+        self.res.delete.assert_called_with(self.sot, error_message=mock.ANY)
 
         self.sot._delete(DeleteableResource, self.fake_id)
-        DeleteableResource.existing.assert_called_with(id=self.fake_id)
-        self.res.delete.assert_called_with(self.session)
+        DeleteableResource.new.assert_called_with(id=self.fake_id)
+        self.res.delete.assert_called_with(self.sot, error_message=mock.ANY)
 
         # Delete generally doesn't return anything, so we will normally
         # swallow any return from within a service's proxy, but make sure
@@ -123,13 +195,14 @@ class TestProxyDelete(testtools.TestCase):
         rv = self.sot._delete(DeleteableResource, self.fake_id)
         self.assertIsNone(rv)
 
-    def test_delete_ResourceNotFound(self):
+    def test_delete_NotFound(self):
         self.res.delete.side_effect = exceptions.NotFoundException(
             message="test", http_status=404)
 
         self.assertRaisesRegex(
-            exceptions.ResourceNotFound,
-            "No %s found for %s" % (DeleteableResource.__name__, self.res),
+            exceptions.NotFoundException,
+            # TODO(shade) The mocks here are hiding the thing we want to test.
+            "test",
             self.sot._delete, DeleteableResource, self.res,
             ignore_missing=False)
 
@@ -141,7 +214,7 @@ class TestProxyDelete(testtools.TestCase):
                           DeleteableResource, self.res, ignore_missing=False)
 
 
-class TestProxyUpdate(testtools.TestCase):
+class TestProxyUpdate(base.TestCase):
 
     def setUp(self):
         super(TestProxyUpdate, self).setUp()
@@ -153,29 +226,28 @@ class TestProxyUpdate(testtools.TestCase):
 
         self.res = mock.Mock(spec=UpdateableResource)
         self.res.update = mock.Mock(return_value=self.fake_result)
-        self.res.update_attrs = mock.Mock()
 
-        self.sot = proxy.BaseProxy(self.session)
+        self.sot = proxy.Proxy(self.session)
 
         self.attrs = {"x": 1, "y": 2, "z": 3}
 
-        UpdateableResource.existing = mock.Mock(return_value=self.res)
-
-    def _test_update(self, value):
-        rv = self.sot._update(UpdateableResource, value, **self.attrs)
-
-        self.assertEqual(rv, self.fake_result)
-        self.res.update_attrs.assert_called_once_with(self.attrs)
-        self.res.update.assert_called_once_with(self.session)
+        UpdateableResource.new = mock.Mock(return_value=self.res)
 
     def test_update_resource(self):
-        self._test_update(self.res)
+        rv = self.sot._update(UpdateableResource, self.res, **self.attrs)
+
+        self.assertEqual(rv, self.fake_result)
+        self.res._update.assert_called_once_with(**self.attrs)
+        self.res.update.assert_called_once_with(self.sot)
 
     def test_update_id(self):
-        self._test_update(self.fake_id)
+        rv = self.sot._update(UpdateableResource, self.fake_id, **self.attrs)
+
+        self.assertEqual(rv, self.fake_result)
+        self.res.update.assert_called_once_with(self.sot)
 
 
-class TestProxyCreate(testtools.TestCase):
+class TestProxyCreate(base.TestCase):
 
     def setUp(self):
         super(TestProxyCreate, self).setUp()
@@ -186,7 +258,7 @@ class TestProxyCreate(testtools.TestCase):
         self.res = mock.Mock(spec=CreateableResource)
         self.res.create = mock.Mock(return_value=self.fake_result)
 
-        self.sot = proxy.BaseProxy(self.session)
+        self.sot = proxy.Proxy(self.session)
 
     def test_create_attributes(self):
         CreateableResource.new = mock.Mock(return_value=self.res)
@@ -196,10 +268,10 @@ class TestProxyCreate(testtools.TestCase):
 
         self.assertEqual(rv, self.fake_result)
         CreateableResource.new.assert_called_once_with(**attrs)
-        self.res.create.assert_called_once_with(self.session)
+        self.res.create.assert_called_once_with(self.sot)
 
 
-class TestProxyGet(testtools.TestCase):
+class TestProxyGet(base.TestCase):
 
     def setUp(self):
         super(TestProxyGet, self).setUp()
@@ -213,26 +285,31 @@ class TestProxyGet(testtools.TestCase):
         self.res.id = self.fake_id
         self.res.get = mock.Mock(return_value=self.fake_result)
 
-        self.sot = proxy.BaseProxy(self.session)
-        RetrieveableResource.existing = mock.Mock(return_value=self.res)
+        self.sot = proxy.Proxy(self.session)
+        RetrieveableResource.new = mock.Mock(return_value=self.res)
 
     def test_get_resource(self):
         rv = self.sot._get(RetrieveableResource, self.res)
 
-        self.res.get.assert_called_with(self.session, args=None)
+        self.res.get.assert_called_with(self.sot, requires_id=True,
+                                        error_message=mock.ANY)
         self.assertEqual(rv, self.fake_result)
 
     def test_get_resource_with_args(self):
-        rv = self.sot._get(RetrieveableResource, self.res, args={'K': 'V'})
+        args = {"key": "value"}
+        rv = self.sot._get(RetrieveableResource, self.res, **args)
 
-        self.res.get.assert_called_with(self.session, args={'K': 'V'})
+        self.res._update.assert_called_once_with(**args)
+        self.res.get.assert_called_with(self.sot, requires_id=True,
+                                        error_message=mock.ANY)
         self.assertEqual(rv, self.fake_result)
 
     def test_get_id(self):
         rv = self.sot._get(RetrieveableResource, self.fake_id)
 
-        RetrieveableResource.existing.assert_called_with(id=self.fake_id)
-        self.res.get.assert_called_with(self.session, args=None)
+        RetrieveableResource.new.assert_called_with(id=self.fake_id)
+        self.res.get.assert_called_with(self.sot, requires_id=True,
+                                        error_message=mock.ANY)
         self.assertEqual(rv, self.fake_result)
 
     def test_get_not_found(self):
@@ -240,47 +317,39 @@ class TestProxyGet(testtools.TestCase):
             message="test", http_status=404)
 
         self.assertRaisesRegex(
-            exceptions.ResourceNotFound,
-            "No %s found for %s" % (RetrieveableResource.__name__, self.res),
-            self.sot._get, RetrieveableResource, self.res)
+            exceptions.NotFoundException,
+            "test", self.sot._get, RetrieveableResource, self.res)
 
 
-class TestProxyList(testtools.TestCase):
+class TestProxyList(base.TestCase):
 
     def setUp(self):
         super(TestProxyList, self).setUp()
 
         self.session = mock.Mock()
 
-        self.fake_a = 1
-        self.fake_b = 2
-        self.fake_c = 3
-        self.fake_resource = resource.Resource.new(id=self.fake_a)
+        self.args = {"a": "A", "b": "B", "c": "C"}
         self.fake_response = [resource.Resource()]
-        self.fake_query = {"a": self.fake_resource, "b": self.fake_b}
-        self.fake_path_args = {"c": self.fake_c}
 
-        self.sot = proxy.BaseProxy(self.session)
+        self.sot = proxy.Proxy(self.session)
         ListableResource.list = mock.Mock()
         ListableResource.list.return_value = self.fake_response
 
-    def _test_list(self, path_args, paginated, **query):
-        rv = self.sot._list(ListableResource, path_args=path_args,
-                            paginated=paginated, **query)
+    def _test_list(self, paginated):
+        rv = self.sot._list(ListableResource, paginated=paginated, **self.args)
 
         self.assertEqual(self.fake_response, rv)
         ListableResource.list.assert_called_once_with(
-            self.session, path_args=path_args, paginated=paginated,
-            params={'a': self.fake_a, 'b': self.fake_b})
+            self.sot, paginated=paginated, **self.args)
 
     def test_list_paginated(self):
-        self._test_list(self.fake_path_args, True, **self.fake_query)
+        self._test_list(True)
 
     def test_list_non_paginated(self):
-        self._test_list(self.fake_path_args, False, **self.fake_query)
+        self._test_list(False)
 
 
-class TestProxyHead(testtools.TestCase):
+class TestProxyHead(base.TestCase):
 
     def setUp(self):
         super(TestProxyHead, self).setUp()
@@ -294,31 +363,30 @@ class TestProxyHead(testtools.TestCase):
         self.res.id = self.fake_id
         self.res.head = mock.Mock(return_value=self.fake_result)
 
-        self.sot = proxy.BaseProxy(self.session)
-        HeadableResource.existing = mock.Mock(return_value=self.res)
+        self.sot = proxy.Proxy(self.session)
+        HeadableResource.new = mock.Mock(return_value=self.res)
 
     def test_head_resource(self):
         rv = self.sot._head(HeadableResource, self.res)
 
-        self.res.head.assert_called_with(self.session)
+        self.res.head.assert_called_with(self.sot)
         self.assertEqual(rv, self.fake_result)
 
     def test_head_id(self):
         rv = self.sot._head(HeadableResource, self.fake_id)
 
-        HeadableResource.existing.assert_called_with(id=self.fake_id)
-        self.res.head.assert_called_with(self.session)
+        HeadableResource.new.assert_called_with(id=self.fake_id)
+        self.res.head.assert_called_with(self.sot)
         self.assertEqual(rv, self.fake_result)
 
-    def test_head_no_value(self):
-        MockHeadResource = mock.Mock(spec=HeadableResource)
-        instance = mock.Mock()
-        MockHeadResource.return_value = instance
 
-        self.sot._head(MockHeadResource)
+class TestProxyWaits(base.TestCase):
 
-        MockHeadResource.assert_called_with()
-        instance.head.assert_called_with(self.session)
+    def setUp(self):
+        super(TestProxyWaits, self).setUp()
+
+        self.session = mock.Mock()
+        self.sot = proxy.Proxy(self.session)
 
     @mock.patch("openstack.resource.wait_for_status")
     def test_wait_for(self, mock_wait):
@@ -326,7 +394,7 @@ class TestProxyHead(testtools.TestCase):
         mock_wait.return_value = mock_resource
         self.sot.wait_for_status(mock_resource, 'ACTIVE')
         mock_wait.assert_called_once_with(
-            self.session, mock_resource, 'ACTIVE', [], 2, 120)
+            self.sot, mock_resource, 'ACTIVE', [], 2, 120)
 
     @mock.patch("openstack.resource.wait_for_status")
     def test_wait_for_params(self, mock_wait):
@@ -334,20 +402,18 @@ class TestProxyHead(testtools.TestCase):
         mock_wait.return_value = mock_resource
         self.sot.wait_for_status(mock_resource, 'ACTIVE', ['ERROR'], 1, 2)
         mock_wait.assert_called_once_with(
-            self.session, mock_resource, 'ACTIVE', ['ERROR'], 1, 2)
+            self.sot, mock_resource, 'ACTIVE', ['ERROR'], 1, 2)
 
     @mock.patch("openstack.resource.wait_for_delete")
     def test_wait_for_delete(self, mock_wait):
         mock_resource = mock.Mock()
         mock_wait.return_value = mock_resource
         self.sot.wait_for_delete(mock_resource)
-        mock_wait.assert_called_once_with(
-            self.session, mock_resource, 2, 120)
+        mock_wait.assert_called_once_with(self.sot, mock_resource, 2, 120)
 
     @mock.patch("openstack.resource.wait_for_delete")
     def test_wait_for_delete_params(self, mock_wait):
         mock_resource = mock.Mock()
         mock_wait.return_value = mock_resource
         self.sot.wait_for_delete(mock_resource, 1, 2)
-        mock_wait.assert_called_once_with(
-            self.session, mock_resource, 1, 2)
+        mock_wait.assert_called_once_with(self.sot, mock_resource, 1, 2)
